@@ -49,7 +49,7 @@ from calibre.utils.icu import character_name, sort_key
 from calibre.utils.localization import localize_user_manual_link
 
 def open_donate():
-    open_url(QUrl('http://calibre-ebook.com/donate'))
+    open_url(QUrl('https://calibre-ebook.com/donate'))
 
 class Central(QStackedWidget):  # {{{
 
@@ -480,7 +480,7 @@ class Main(MainWindow):
                 'Close all tabs except the current tab'))
         self.action_help = treg(
             'help.png', _('User &Manual'), lambda : open_url(QUrl(localize_user_manual_link(
-                'http://manual.calibre-ebook.com/edit.html'))), 'user-manual', 'F1', _(
+                'https://manual.calibre-ebook.com/edit.html'))), 'user-manual', 'F1', _(
                 'Show User Manual'))
         self.action_browse_images = treg(
             'view-image.png', _('&Browse images in book'), self.boss.browse_images, 'browse-images', (), _(
